/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

                   /* SET TABS TO 4 TO READ THESE SOURCES */


#include "xdipo.h"

FD_xdipo *fdui;


#define DISEQC_DELAY	15


void set_ts_filt(int fd, uint16_t pid, dmx_pes_type_t pestype)
{
struct dmx_pes_filter_params pesFilterParams;

if(debug_flag)
	{
	fprintf(stderr, "set_ts_filt(): arg fd=%d pid=%d pestype=%d\n", fd, pid, pestype);
	}

pesFilterParams.pid     = pid;
pesFilterParams.input   = DMX_IN_FRONTEND;
pesFilterParams.output  = DMX_OUT_TS_TAP;
pesFilterParams.pes_type = pestype;
pesFilterParams.flags   = DMX_IMMEDIATE_START;

if(ioctl(fd, DMX_SET_PES_FILTER, &pesFilterParams) < 0) 
	{
	fprintf(stderr, "set_ts_filt(): DMX_SET_PES_FILTER failed for pid=%d: ", pid);
	perror("reason ");
	}

} /* end function set_ts_filt */


static inline void msleep(uint32_t msec)
{
struct timespec req = { msec / 1000, 1000000 * (msec % 1000) };

while (nanosleep(&req, &req) ) ;
} /* end function msleep */


int send_diseqc_command(int *data, int length)
{
int a, i;
struct dvb_diseqc_master_cmd *cmd;
int voltage;
int tone_flag, highband_flag;
struct dvb_frontend_parameters feparams;
#ifdef TEST_CODE
struct dvb_frontend_info fe_info;
#endif // TEST_CODE
unsigned long freq, srate;
char temp[80];
char temp2[20];

if(debug_flag)
	{
	fprintf(stderr,\
	"send_diseqc_command(): arg data=%p length=%d using fd_frontend=%d\n",\
	data, length, fd_frontend);

	fprintf(stderr, "sending: ");	
	for(i = 0; i < length; i++) fprintf(stderr, "%02x ", data[i]);
	fprintf(stderr, "\n");
	}

if(length > 6)
	{
	fprintf(stderr, "send_diseqc_command(): length %d longer then 6, error.\n", length);

	return 0;
	}

#ifdef TEST_CODE
/* get frontend info */
a = ioctl(fd_frontend, FE_GET_INFO, &fe_info);
if(a)
	{
	fprintf(stderr, "send_diseqc_command(): FE_GET_INFO failed.\n");
	perror("reason ");
	return 0;
	}

/* print all we can find out about this front end
struct dvb_frontend_info {
	char       name[128];
	fe_type_t  type;
	uint32_t   frequency_min;
	uint32_t   frequency_max;
	uint32_t   frequency_stepsize;
	uint32_t   frequency_tolerance;
	uint32_t   symbol_rate_min;
	uint32_t   symbol_rate_max;
	uint32_t   symbol_rate_tolerance;     /\u22c6 ppm \u22c6/
	uint32_t   notifier_delay;            /\u22c6 ms \u22c6/
	fe_caps_t  caps;
	};

typedef enum fe_caps {
	FE_IS_STUPID                  = 0,
	FE_CAN_INVERSION_AUTO         = 0x1,
	FE_CAN_FEC_1_2                = 0x2,
	FE_CAN_FEC_2_3                = 0x4,
	FE_CAN_FEC_3_4                = 0x8,
	FE_CAN_FEC_4_5                = 0x10,
	FE_CAN_FEC_5_6                = 0x20,
	FE_CAN_FEC_6_7                = 0x40,
	FE_CAN_FEC_7_8                = 0x80,
	FE_CAN_FEC_8_9                = 0x100,
	FE_CAN_FEC_AUTO               = 0x200,
	FE_CAN_QPSK                   = 0x400,
	FE_CAN_QAM_16                 = 0x800,
	FE_CAN_QAM_32                 = 0x1000,
	FE_CAN_QAM_64                 = 0x2000,
	FE_CAN_QAM_128                = 0x4000,
	FE_CAN_QAM_256                = 0x8000,
	FE_CAN_QAM_AUTO               = 0x10000,
	FE_CAN_TRANSMISSION_MODE_AUTO = 0x20000,
	FE_CAN_BANDWIDTH_AUTO         = 0x40000,
	FE_CAN_GUARD_INTERVAL_AUTO    = 0x80000,
	FE_CAN_HIERARCHY_AUTO         = 0x100000,
	FE_CAN_8VSB                   = 0x200000,
	FE_CAN_16VSB                  = 0x400000,
	FE_HAS_EXTENDED_CAPS          = 0x800000,
	FE_CAN_TURBO_FEC              = 0x8000000,
	FE_CAN_2G_MODULATION          = 0x10000000,
	FE_NEEDS_BENDING              = 0x20000000,
	FE_CAN_RECOVER                = 0x40000000,
	FE_CAN_MUTE_TS                = 0x80000000
	} fe_caps_t;
*/

fprintf(stderr, "fe_info.type=%d\n", fe_info.type);

fprintf(stderr, "fe_info.frequency_min=%u\n", fe_info.frequency_min);
fprintf(stderr, "fe_info.frequency_max=%u\n", fe_info.frequency_max);
fprintf(stderr, "fe_info.frequency_stepsize=%u\n", fe_info.frequency_stepsize);
fprintf(stderr, "fe_info.frequency_tolerance=%u\n", fe_info.frequency_tolerance);
fprintf(stderr, "fe_info.symbol_rate_min=%u\n", fe_info.symbol_rate_min);
fprintf(stderr, "fe_info.symbol_rate_max=%u\n", fe_info.symbol_rate_max);
fprintf(stderr, "fe_info.symbol_rate_tolerance=%u\n", fe_info.symbol_rate_tolerance);
fprintf(stderr, "fe_info.notifier_delay]=%u\n", fe_info.notifier_delay);
fprintf(stderr, "fe_info.caps=%x\n", fe_info.caps);

uint32_t ua;
ua = fe_info.caps;


if(ua & FE_IS_STUPID)					fprintf(stderr, "can do FE_IS_STUPID\n");
if(ua & FE_CAN_INVERSION_AUTO)			fprintf(stderr, "can do	FE_CAN_INVERSION_AUTO\n");
if(ua & FE_CAN_FEC_1_2)					fprintf(stderr, "can do	FE_CAN_FEC_1_2\n");
if(ua & FE_CAN_FEC_2_3)					fprintf(stderr, "can do	FE_CAN_FEC_2_3\n");
if(ua & FE_CAN_FEC_3_4)					fprintf(stderr, "can do	FE_CAN_FEC_3_4\n");
if(ua & FE_CAN_FEC_4_5)					fprintf(stderr, "can do	FE_CAN_FEC_4_5\n");
if(ua & FE_CAN_FEC_5_6)					fprintf(stderr, "can do	FE_CAN_FEC_5_6\n");
if(ua & FE_CAN_FEC_6_7)					fprintf(stderr, "can do	FE_CAN_FEC_6_7\n");
if(ua & FE_CAN_FEC_7_8)					fprintf(stderr, "can do	FE_CAN_FEC_7_8\n");
if(ua & FE_CAN_FEC_8_9)					fprintf(stderr, "can do	FE_CAN_FEC_8_9\n");
if(ua & FE_CAN_FEC_AUTO)				fprintf(stderr, "can do	FE_CAN_FEC_AUTO\n");
if(ua & FE_CAN_QPSK)					fprintf(stderr, "can do	FE_CAN_QPSK\n");
if(ua & FE_CAN_QAM_16)					fprintf(stderr, "can do	FE_CAN_QAM_16\n");
if(ua & FE_CAN_QAM_32)					fprintf(stderr, "can do	FE_CAN_QAM_32\n");
if(ua & FE_CAN_QAM_64)					fprintf(stderr, "can do	FE_CAN_QAM_64\n");
if(ua & FE_CAN_QAM_128)					fprintf(stderr, "can do	FE_CAN_QAM_128\n");
if(ua & FE_CAN_QAM_256)					fprintf(stderr, "can do	FE_CAN_QAM_256\n");
if(ua & FE_CAN_QAM_AUTO)				fprintf(stderr, "can do	FE_CAN_QAM_AUTO\n");
if(ua & FE_CAN_TRANSMISSION_MODE_AUTO)	fprintf(stderr, "can do	FE_CAN_TRANSMISSION_MODE_AUTO\n");
if(ua & FE_CAN_BANDWIDTH_AUTO)			fprintf(stderr, "can do	FE_CAN_BANDWIDTH_AUTO\n");
if(ua & FE_CAN_GUARD_INTERVAL_AUTO)		fprintf(stderr, "can do	FE_CAN_GUARD_INTERVAL_AUTO\n");
if(ua & FE_CAN_HIERARCHY_AUTO)			fprintf(stderr, "can do	FE_CAN_HIERARCHY_AUTO\n");
if(ua & FE_CAN_8VSB)					fprintf(stderr, "can do	FE_CAN_8VSB\n");
if(ua & FE_CAN_16VSB)					fprintf(stderr, "can do	FE_CAN_16VSB\n");
if(ua & FE_HAS_EXTENDED_CAPS)			fprintf(stderr, "can do	FE_HAS_EXTENDED_CAPS\n");
if(ua & FE_CAN_TURBO_FEC)				fprintf(stderr, "can do	FE_CAN_TURBO_FEC\n");
if(ua & FE_CAN_2G_MODULATION)			fprintf(stderr, "can do	FE_CAN_2G_MODULATION\n");
if(ua & FE_NEEDS_BENDING)				fprintf(stderr, "can do	FE_NEEDS_BENDING\n");
if(ua & FE_CAN_RECOVER)					fprintf(stderr, "can do	FE_CAN_RECOVER\n");
if(ua & FE_CAN_MUTE_TS)					fprintf(stderr, "can do	FE_CAN_MUTE_TS\n");

#endif	// TEST_CODE

#define SLOF (11700*1000UL)
#define LOF1 (9750*1000UL)
#define LOF2 (10600*1000UL)

/* if pol is vertical then voltage is 13V, if horizontal then 18V */
/* if band is low then tone is off, else tone is on */

freq = frequency * 1000UL;

srate = symbolrate *1000UL;

if(debug_flag)
	{
	fprintf(stderr, "freq=%lu\n", freq);

	fprintf(stderr, "setting frontend:\n\
	freq=%lu polarization=%c inversion=%d srate=%lu fec_inner=%d modulation=%d\n",\
	freq, polarization, INVERSION_OFF, srate, FEC_AUTO, QAM_64);
	}

if(freq > SLOF)
	{
	freq -= LOF2;
	highband_flag = 1;
	}
else
	{
	freq -= LOF1;
	highband_flag = 0;
	}


/*
struct dvb_frontend_parameters {
uint32_t frequency;			(absolute) frequency in Hz for QAM/OF
							intermediate frequency in kHz for QPS
fe_spectral_inversion_t inversion;
union {
	struct dvb_qpsk_parameters qpsk;
	struct dvb_qam_parameters qam;
	struct dvb_ofdm_parameters ofdm;
	} u;
};
                                                                                                                  
struct dvb_qpsk_parameters {
uint32_t               symbol_rate;   symbol rate in Symbols per second
fe_code_rate_t fec_inner;            forward error correction (see abo
};

for cable QAM frontend you use the QAM parameters structure
struct dvb_qam_parameters {
uint32_t		symbol_rate;		symbol rate in Symbols per second
fe_code_rate_t	fec_inner;			forward error correction (see abo
fe_modulation_t modulation;			 modulation type (see above)
};
*/

feparams.frequency = freq;
//feparams.inversion = INVERSION_OFF;
feparams.inversion = INVERSION_AUTO;

//#ifdef NOT_FOR_TEST
feparams.u.qam.symbol_rate = srate; 
//feparams.u.qam.modulation = QAM_64;
//feparams.u.qam.modulation = QPSK;
//#endif // NOT_FOR_TEST

//fprintf(stderr, "WAS srate=%lu\n", srate);
//fprintf(stderr, "WAS freq=%lu\n", freq);

//feparams.u.qpsk.symbol_rate = 23000000;
//feparams.u.qpsk.fec_inner = FEC_AUTO;
//feparams.u.qpsk.fec_inner = FEC_8_9;

//fprintf(stderr, "WAS srate=%lu\n", srate);
//fprintf(stderr, "WAS freq=%lu\n", freq);



if( toupper(polarization) == 'V') voltage = SEC_VOLTAGE_13; else voltage = SEC_VOLTAGE_18; 
if( highband_flag) tone_flag = SEC_TONE_ON; else tone_flag = SEC_TONE_OFF;

/* if a diseqc command requested */
if(length)
	{
	/* no tone, we are going to send pulses */
	a = ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): set tone failed.\n");
		perror("reason ");

		return 0;
		}

	/* Mosteck SG 2100 A only moves with 13 V?? */
	a = ioctl(fd_frontend, FE_SET_VOLTAGE, SEC_VOLTAGE_13);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): set voltage failed.\n");
		perror("reason ");

		return 0;
		}

	msleep(15);

//struct dvb_diseqc_master_cmd
//{
//uint8_t msg [6];        /*  { framing, address, command, data [3] } */
//uint8_t msg_len;        /*  valid values are 3...6  */
//};

	cmd = malloc(sizeof(struct dvb_diseqc_master_cmd) );
	if(! cmd)
		{
		fprintf(stderr, "send_diseqc_command(): malloc struct dvb_diseqc_master_cmd failed.\n");

		return 0;
		}

	if(debug_flag)
		{
		fprintf(stderr, "send_diseqc_command(): cmd=%p\n", cmd);
		}

	sprintf(temp, "f=%.2f p=%c s=%d command=", frequency, polarization, symbolrate);

	for(i = 0; i < length; i++)	
		{
		cmd -> msg[i] = (int8_t) data[i];

		sprintf(temp2, "%02x ", cmd -> msg[i]);

		strcat(temp, temp2);	
		} /* end for all bytes in diseqc command */

	if(gui_flag)
		{
		fl_set_object_label(fdui -> main_form_command_output_field, temp);
		}

	cmd -> msg_len = length;

	a = ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, cmd);
	free(cmd);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): send master command failed.\n");
		perror("reason ");

		return 0;
		}

	usleep(200000);
	} /* end if length (any diseqc command) */

/* set the required voltage for polarization */
a = ioctl(fd_frontend, FE_SET_VOLTAGE, voltage);
if(a)
	{
	fprintf(stderr, "send_diseqc_command(): set voltage failed.\n");
	perror("reason ");

	return 0;
	}

/* restore the 22 kHz tone if it was requested for high band */
a = ioctl(fd_frontend, FE_SET_TONE, tone_flag);
if(a)
	{
	fprintf(stderr, "send_diseqc_command(): set tone failed.\n");
	perror("reason ");
	return 0;
	}

#define NEW_API

#ifdef NEW_API
/* DVB-S2 mod here ? */

//enum fe_delivery_system delivery_system;

//fprintf(stderr, "SYS_DVBS=%d\n", SYS_DVBS);

int delivery_system;
if(strcmp(type, "S") == 0) delivery_system = SYS_DVBS;
else if(strcmp(type, "S2") == 0) delivery_system = SYS_DVBS2;
else
	{
	fprintf(stderr, "send_diseqc_command(): invalid deliviery system.\n");

	return 0;
	}

/*
static struct t_channel_parameter_map modulation_values[] = {
 // {   0, NONE,    "none" },
 // {   4, QAM_4,    "QAM4" },
  {  16, QAM_16,   "QAM16" },
  {  32, QAM_32,   "QAM32" },
  {  64, QAM_64,   "QAM64" },
  { 128, QAM_128,  "QAM128" },
  { 256, QAM_256,  "QAM256" },
//  { 512, QAM_512,  "QAM512" },
//  {1024, QAM_1024, "QAM1024" },
//  {   1, BPSK,    "BPSK" },
  {   2, QPSK,    "QPSK" },
//  {   3, OQPSK,   "OQPSK" },
  {   5, PSK_8,    "8PSK" },
  {   6, APSK_16,  "16APSK" },
//  {   7, _32APSK,  "32APSK" },
//  {   8, OFDM,    "OFDM" },
//  {   9, COFDM,   "COFDM" },
  {  10, VSB_8,    "VSB8" },
  {  11, VSB_16,   "VSB16" },
  { 998, QAM_AUTO, "QAMAUTO" },
//  { 999, AUTO },
  { -1 }
  };
*/

int modulation_system;
if(strcmp(modulation, "BPSK") == 0) modulation_system = 1;
else if(strcmp(modulation, "QPSK") == 0) modulation_system = QPSK;
else if(strcmp(modulation, "8PSK") == 0) modulation_system = PSK_8;
else
	{
	fprintf(stderr, "send_diseqc_command(): invalid modulation system.\n");

	return 0;
	}


/*
static struct t_channel_parameter_map coderate_values[] = {
  {   0, FEC_NONE, "none" },
  {  12, FEC_1_2,  "1/2" },
//  {  13, FEC_1_3,  "1/3" },
//  {  14, FEC_1_4,  "1/4" },
  {  23, FEC_2_3,  "2/3" },
//  {  25, FEC_2_5,  "2/5" },
  {  34, FEC_3_4,  "3/4" },
  {  35, FEC_3_5,  "3/5" },
  {  45, FEC_4_5,  "4/5" },
  {  56, FEC_5_6,  "5/6" },
  {  67, FEC_6_7,  "6/7" },
  {  78, FEC_7_8,  "7/8" },
  {  89, FEC_8_9,  "8/9" },
  { 910, FEC_9_10, "9/10" },
  { 999, FEC_AUTO, "auto" },
  { -1 }
  };
*/


int tfec;
if(strcasecmp(fec, "AUTO") == 0) tfec = FEC_AUTO;
else if(strcmp(fec, "1/2") == 0) tfec = FEC_1_2;
else if(strcmp(fec, "2/3") == 0) tfec = FEC_2_3;
else if(strcmp(fec, "3/4") == 0) tfec = FEC_3_4;
else if(strcmp(fec, "3/5") == 0) tfec = FEC_3_5;
else if(strcmp(fec, "4/5") == 0) tfec = FEC_4_5;
else if(strcmp(fec, "5/6") == 0) tfec = FEC_5_6;
else if(strcmp(fec, "6/7") == 0) tfec = FEC_6_7;
else if(strcmp(fec, "7/8") == 0) tfec = FEC_7_8;
else if(strcmp(fec, "8/9") == 0) tfec = FEC_8_9;
else if(strcmp(fec, "9/10") == 0) tfec = FEC_9_10;
else
	{
	fprintf(stderr, "send_diseqc_command(): invalid fec.\n");
	
	return 0;
	} 

/*
static struct t_channel_parameter_map rolloff_values[] =
	{
//	{   0, ROLLOFF_AUTO, "auto"},
	{  20, ROLLOFF_20, "0.20" },
	{  25, ROLLOFF_25, "0.25" },
	{  35, ROLLOFF_35, "0.35" },
	{ -1 }
	};
*/

int trolloff;
if(rolloff == 0.0) trolloff = ROLLOFF_AUTO;
else if(rolloff == 0.20) trolloff = ROLLOFF_20;
else if(rolloff == 0.25) trolloff = ROLLOFF_25;
else if(rolloff == 0.35) trolloff = ROLLOFF_35;
else
	{
	fprintf(stderr, "send_diseqc_command(): invalid rolloff.\n");
	
	return 0;
	}

int tsymbolrate;
tsymbolrate = symbolrate * 1000;

struct dtv_property p1[] =
	{
	{ .cmd = DTV_CLEAR },
	};

struct dtv_properties cmdseq1 =
	{
	.num = 1,
	.props = p1
	};

a = ioctl(fd_frontend, FE_SET_PROPERTY, &cmdseq1);
if(a == -1) 
	{
	fprintf(stderr, "send_diseqc_command(): ioctl(fd_frontend, FE_SET_PROPERTY, DTV_CLEAR) failed\n");
	perror("reason ");
	
	return 0;
	}

struct dtv_property p[] =
	{
	{ .cmd = DTV_DELIVERY_SYSTEM,	.u.data = delivery_system },
	{ .cmd = DTV_FREQUENCY,			.u.data = freq },
	{ .cmd = DTV_MODULATION,		.u.data = modulation_system },
	{ .cmd = DTV_SYMBOL_RATE,		.u.data = tsymbolrate  },
	{ .cmd = DTV_INNER_FEC,			.u.data = tfec },
	{ .cmd = DTV_INVERSION,			.u.data = INVERSION_AUTO },
	{ .cmd = DTV_ROLLOFF,			.u.data = trolloff },
	{ .cmd = DTV_PILOT,				.u.data = PILOT_AUTO },
	{ .cmd = DTV_TUNE },
	};

struct dtv_properties cmdseq =
	{
	.num = 9,
	.props = p
	};

struct dvb_frontend_event ev;

/* discard stale QPSK events */
while(1)
	{
	a = ioctl(fd_frontend, FE_GET_EVENT, &ev);
	if(a == -1)  break;
	}

if(debug_flag)
	{
	fprintf(stderr, "send_diseqc_command(): FE_SET_PROPERTY: delivery_system=%d freq=%lu modulation_system=%d tsymbolrate=%u tfec=%d trolloff=%d\n",\
	delivery_system, freq, modulation_system, tsymbolrate, tfec, trolloff);	
	}

a = ioctl(fd_frontend, FE_SET_PROPERTY, &cmdseq);
if(a == -1)
	{
	fprintf(stderr, "send_diseqc_command(): ioctl(fd_frontend, FE_SET_PROPERTY, &cmdseq) failed\n");
    perror("reason ");

	return 0;
	}                   

#else // OLD API
a = ioctl(fd_frontend, FE_SET_FRONTEND, &feparams);
if(a < 0)
	{
	fprintf(stderr, "send_diseqc_command(): set frontend failed.\n");
	perror("reason ");	

	return 0;
	}
#endif // OLD API




return 1;
} /* end function send_diseqc_command */


int get_frontend_status()
{
int a;
fe_status_t fe_status;

if(debug_flag)
	{
	fprintf(stderr, "get_frontend_status(): arg none, using fd_frontend=%d\n", fd_frontend);
	}

a = ioctl(fd_frontend, FE_READ_STATUS, &fe_status);
if(a)
    {
    fprintf(stderr, "test_lock():  FE_GET_STATUS failed.\n");
    perror("reason ");

    return 0;
    }

if(fe_status & FE_HAS_SIGNAL) carrier_flag = 1;
else carrier_flag = 0;

if(fe_status & FE_HAS_VITERBI) viterbi_flag = 1;
else viterbi_flag = 0;

if(fe_status & FE_HAS_SYNC) sync_flag = 1;
else sync_flag = 0;

if(fe_status & FE_HAS_LOCK) lock_flag = 1;
else lock_flag = 0;

if(debug_flag)
	{
	fprintf(stderr,\
	"get_status() carrier_flag=%d viterbi_flag=%d sync_flag=%d lock_flag=%d\n",\
	carrier_flag, viterbi_flag, sync_flag, lock_flag);
	}

if(gui_flag)
	{
	if(carrier_flag)
		{
		fl_set_object_color(fdui -> main_form_carrier_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_carrier_output_field, FL_RED, FL_GREEN);
		}

	if(viterbi_flag)
		{
		fl_set_object_color(fdui -> main_form_viterbi_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_viterbi_output_field, FL_RED, FL_GREEN);
		}

	if(sync_flag)
		{
		fl_set_object_color(fdui -> main_form_sync_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_sync_output_field, FL_RED, FL_GREEN);
		}

	if(lock_flag)
		{
		fl_set_object_color(fdui -> main_form_lock_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_lock_output_field, FL_RED, FL_GREEN);
		}

	} /* end if gui_flag */

return 1;
} /* end function get_frontend_status */

