/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

                   /* SET TABS TO 4 TO READ THESE SOURCES */


#include "xdipo.h"

FD_xdipo *fdui;


/* modify this line if you prefer a different default language */
#define DEFAULT_LANGUAGE		"eng"


#define PROCESS_PACKETS			1000
#define CSA_PACKETS				70


#define MAX_PMT_SIZE			4096	// really have no idea, anyone?
#define MAX_SUBTITLE_CHANNELS	256		// ?
#define MAX_AUDIO_CHANNELS		256		// ?
#define MAX_TELETEXT_CHANNELS	256		// ?
#define MAX_AC3_CHANNELS		256		// ?


unsigned char pmt_buffer[MAX_PMT_SIZE];



struct ECMINFO
{
  struct ECMINFO *next;
  char *name;
  int system;
  int ecm_pid;
  int id;
};


static struct ECMINFO *ecminfos = 0;
static struct ECMINFO *pa;


#define SECA_CA_SYSTEM			0x0100
#define VIACCESS_CA_SYSTEM		0x0500
#define IRDETO_CA_SYSTEM		0x0600
#define XXX_SYSTEM				0x0900 // 09 19 ???
#define CONAX_CA_SYSTEM			0x0b00
#define NAGRA_BEV_SYSTEM		0x1234
#define BETA_CA_SYSTEM			0x1700
#define NAGRA_CA_SYSTEM			0x1800
#define SHL_SYSTEM				0x4A00
#define CRYPTOWORKS_SYSTEM		0x0D00



void ParseCADescriptor (uint8_t *data, int length)
{
static char *seca_name = "Seca";
static char *viaccess_name = "Viaccess";
static char *nagra_name = "Nagra";
static char *irdeto_name = "Irdeto";
static char *conax_name = "Conax";

int ca_system;
int j;

struct ECMINFO *e;

//fprintf(stderr, "POSITION=%d\n", data - pmt_buffer);

if(debug_flag)
	{
//    fprintf(stderr,"ParseCADescriptor(): arg *data=%p length=%d\n", data, length);

//    for(j = 0; j < length; j++) fprintf(stderr," %02d", j);
//	fprintf(stderr,"\n");
//   for(j = 0; j < length; j++) fprintf(stderr," %02x", data[j]);
//    fprintf(stderr,"\n");

	fprintf(stderr, "pmt_buffer:\n");
	for(j = 0; j < 25; j++) fprintf(stderr, "%03d ", j + (int)(data - pmt_buffer) );
	fprintf(stderr, "\n");
	for(j = 0; j < 25; j++) fprintf(stderr, " %02x ", pmt_buffer[j + (int)(data - pmt_buffer) ]);
	fprintf(stderr, "\n");
	}


/* only test the upper 8 bits of the ca_system */
ca_system = (data[0] << 8) /* | data[1] */;

//fprintf(stderr, "length=%d\n", length);

if(debug_flag)
	{
	fprintf(stderr, "CA system=0x%04x\n",  (data[0] << 8) | data[1]);
 	}

// lincardemu:
// 0x71 00 seca1?
// 0x10 00 seca2?

switch(ca_system)
	{
	case SECA_CA_SYSTEM:
    	for(j = 2; j < length; j += 15)
			{
			e = malloc(sizeof(struct ECMINFO));
			if (e == 0) return;
			e->system = ca_system;
			e->name = seca_name;
			e->ecm_pid = ((data[j] & 0x1f) << 8) | data[j+1];
			e->id = (data[j+2] << 8) | data[j+3];
			e->next = ecminfos;
			ecminfos = e;
			fprintf(stderr, "Found seca id %04x\n", e->id);
			}        
		break;

	case VIACCESS_CA_SYSTEM:
		j = 4;
		while(j < length)
			{
			if(data[j] == 0x14)
				{
				e = malloc(sizeof(struct ECMINFO));
				if (e == 0) return;
				e->system = ca_system;
				e->name = viaccess_name;
				e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
				e->id = (data[j + 2] << 16) | (data[j + 3] << 8) | (data[j + 4] & 0xf0);
				e->next = ecminfos;
				ecminfos = e;
				fprintf(stderr, "Found viaccess id %04x\n", e->id);
				}

			j += 2 + data[j + 1];
			}
		break;
	case IRDETO_CA_SYSTEM:
	case BETA_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = irdeto_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found irdeto\n");
		break;
	case NAGRA_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = nagra_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found nagra\n");
		break;
	case CONAX_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = conax_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found conax\n");
		break;
	default:
		fprintf(stderr, "jpinfo: unknown ca_system 0x%02x\n", ca_system);
		break;
	} /* end switch ca_system */

} /* end function ParseCADescriptor */


int process_stream(int file_fd, int service_id)
{
int a, i;
struct ECMINFO *e;
int parity;
unsigned long packets, count;
int packets_in, packets_out;
unsigned char *inptr;
int pid;
int payload_unit_start_indicator;
int pmt_length = 0;
unsigned char *pmtbptr = 0;
int length = 0;
int info_len, data_len, index;
int pnr = -1;
int section_number;
int last_section_number;
int current_next_indicator;
int pcr_pid;
int program_info_length;
int qpid = 0; // -Wall
int have_reported_success_flag;
int loop_cnt = 0;


int ecm_pid = 0; // -Wall
int pmt_pid = 0; // -Wall
int video_pid = 0; // -Wall

int hbbtv_pid;

int subtitle_pid[MAX_SUBTITLE_CHANNELS];
int subtitle_channels;

int audio_pid[MAX_AUDIO_CHANNELS];
int audio_channels;
int selected_audio_pid = 0; // -Wall

int teletext_pid = 0; // -Wall
int ac3_pid = 0; // -Wall

char language[4];

int have_pids_flag = 0;

if(debug_flag)
	{
	fprintf(stderr, "process_stream(): arg file_fd=%d service_id=%d\n", file_fd, service_id);
	}

e = malloc(sizeof(struct ECMINFO));
if(e == 0) return 0;

e->ecm_pid = ecm_pid;
e->next = ecminfos;
ecminfos = e;

e = ecminfos;

have_reported_success_flag = 0;
parity = 0xff;
packets = 0;
count = 0;
packets_out = 0;
while(1)
	{
	// ssize_t read(int fd, void *buf, size_t count);
//	a = read(file_fd, input_buffer, process_packets * 188);
	a = read(file_fd, input_buffer, 188);

	if(a == -1)
		{
		if(errno == EAGAIN)
			{
			usleep(100000);

			continue;
			}

		if(debug_flag)
			{
			fprintf(stderr, "process_stream(): read() returned -1 returning.\n"); 
			perror("reason ");
			}
			
		return 0;
		}

//	if(a != process_packets * 188) 
	if(a != 188) 
		{
		if(debug_flag)
			{
			fprintf(stderr, "process_stream(): could read only %d of %d packets, returning.\n", a, process_packets * 188);
			}
			
		return 0;
		}

	packets_in = 0;
	inptr = input_buffer;
	while(1)
		{
		if(debug_flag)
			{
			fprintf(stderr, "inptr[0]=0x%02x\n", inptr[0]);
			}
			
		if(inptr[0] == 0x47)
			{
			pid = ( (inptr[1] & 0x1f) << 8) | inptr[2];

			if(debug_flag)
				{
				fprintf(stderr, "pid=%d\n", pid);
				}

/*
we have 4 bytes:
8	bits	0x47 sync byte
1	bit		transport error flag
1	bit		payload unit start indicator
1	bit		transport priority
13	bits	PID
2	bits	scrambling control
2	bits	adaption field control
4	bits	continuity counter
?	bits	possible adaption field	
*/


//#define TRACE_PACKETS
#ifdef TRACE_PACKETS
trerr = (inptr[1] & 0x80) >> 7;
pstrt = (inptr[1] & 0x40) >> 6;
prio = (inptr[1] & 0x20) >> 5;
//pid = ((inptr[1] & 0x1f) << 8) | inptr[2];
src = (inptr[3] & 0xc0) >> 6;
adap = (inptr[3] & 0x20) >> 4;
cont = inptr[3] & 0x0f;

fprintf(stderr,\
"%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x trerr=%d pstrt=%d prio=%d pid=%d src=%d adap=%d cont=%d\n",\
inptr[0], inptr[1], inptr[2], inptr[3], inptr[4], inptr[5], inptr[6], inptr[7] ,inptr[8], inptr[9],\
trerr, pstrt, prio, pid, src, adap, cont);

#endif
			if(! have_pids_flag)
				{
				if(! pmt_pid)
					{
					if(pid == 0) // PAT, program number gives PMT pid
						{
						int seclen;
						int p;

						if(debug_flag)
							{
							fprintf(stderr, "pid 0, service_id=%d\n", service_id);
							}
						
						/* The PAT is supposed to fit in a 184 bytes packet */

						seclen = ( (inptr[1 + 5] & 0x0f) << 8) | (inptr[2 + 5] & 0xff);
						if(debug_flag)
							{
							fprintf(stderr, "PAT seclen=%d\n", seclen);
							}

						seclen += 3;
						if(debug_flag)
							{
							fprintf(stderr, "Read %d bytes - Found %d services\n", seclen, (seclen - 11) / 4);
							}

						i = 8;
						while(i < seclen - 4)
							{
							p = (inptr[i + 5] << 8) | inptr[i + 1 + 5];

							if(debug_flag)
								{
								fprintf(stderr, "found service id %d\n", p);
								}

							if(p == service_id)
								{
//								if(debug_flag)
									{
									fprintf(stderr, "\n");
									fprintf(stderr, "found service_id %d\n", p);
									}
								
								pmt_pid = ( (inptr[i + 2 + 5] & 0x1f) << 8) | inptr[i + 3 + 5];

								break;
								}

							i += 4;
							} /* end while */

						if(pmt_pid == 0)
							{
//							if(debug_flag)
								{
								fprintf(stderr, "jpinfo: service_id %d not in PAT, stop.\n", service_id);
								}
								
							return 0;
							}
						else
							{
//							if(debug_flag)
								{

								fprintf(stderr, "found PMT pid %d\n", pmt_pid);
								}

							add_pid(pmt_pid);
							}

						} /* end if pid == 0, PAT */
					} /* end if ! pmt_pid */
				else /* have pmt_pid */
					{
					if(pid == pmt_pid)
						{
						// PMT gives ECM pid

						/* we will be here again if we do not find an ecm pid in this packet */

						if(debug_flag)
							{
							fprintf(stderr, "pid = pmt_pid=%d\n", pmt_pid);
							}
							
						/* test if first PMT packet (payload unit start indicator set) */

						payload_unit_start_indicator = (inptr[1] & 0x40) >> 6;

						if(debug_flag)
							{
							fprintf(stderr, "payload_unit_start_indicator=%d\n", payload_unit_start_indicator);
							}

						if(payload_unit_start_indicator) /* first packet of PMT */
							{
							subtitle_channels = 0;
							audio_channels = 0;

							pmt_length = ( (inptr[1 + 5] & 0x0f) << 8) | (inptr[2 + 5] & 0xff);
							if(debug_flag)
								{
								fprintf(stderr, "PMT section length=%d\n", pmt_length);
								}
							
							length = pmt_length;

							/* if length > 184  then the PMT extends over more then one packet */

							pnr = (inptr[3 + 5] << 8) + inptr[4 + 5];
							if(debug_flag)fprintf(stderr, "pnr=%d\n", pnr);

							current_next_indicator = inptr[5 + 5] & 0x1;
							if(debug_flag) fprintf(stderr, "PMT current_next_indicator=%d\n", current_next_indicator);

							section_number = inptr[6 + 5];
							if(debug_flag) fprintf(stderr, "PMT section_number=%d\n", section_number);						

							last_section_number = inptr[7 + 5];
							if(debug_flag) fprintf(stderr, "PMT last_section_number=%d\n", last_section_number);						

							pcr_pid = (inptr[8 + 5] & 0x1f << 8) | (inptr[9 + 5] & 0xff);
							if(debug_flag) fprintf(stderr, "PMT pcr_pid=%d\n", pcr_pid);	

							program_info_length = (inptr[10 + 5] & 0x0f << 8) | (inptr[11 + 5] & 0xff);
							if(debug_flag) fprintf(stderr, "PMT program_info_length=%d\n", program_info_length);

							/* point to start PMT buffer */
							pmtbptr = pmt_buffer;
							if(pmt_length > 188 - 5) a = 188 - 5;
							else a = pmt_length;

							memcpy(pmt_buffer, inptr + 5, a);
							pmtbptr += a;
							pmt_length -= a;
							} /* end if payload_unit_start_indicator */
						else /* not first packet of PMT */
							{
							if(pmt_length > 184) a = 184;
							else a = pmt_length;

							memcpy(pmtbptr, inptr + 4, a);
							pmtbptr += a;
							pmt_length -= a;
							}

						if(debug_flag)
							{
							fprintf(stderr, "pmt_length=%d\n", pmt_length);
							}

						if(pmt_length == 0) // all read
							{
							index = 10;

							info_len = ((pmt_buffer[index] & 0x0F) << 8) + pmt_buffer[index + 1];

							if(debug_flag)
								{
								fprintf(stderr, "PMT info_len=%d\n", info_len);
								}

							index += 2;

							while(info_len > 0)
								{
								if (pmt_buffer[index] == 0x09)
									{
									ParseCADescriptor(&pmt_buffer[index + 2], pmt_buffer[index + 1]);
									}

								info_len -= 2 + pmt_buffer[index + 1];
								index += 2 + pmt_buffer[index + 1];
								}

							while(index < length - 4)
								{
								qpid = ((pmt_buffer[index + 1] & 0x1f) << 8) + pmt_buffer[index + 2];
								if(debug_flag) fprintf(stderr, "qpid=%d ", qpid);
								if(debug_flag) fprintf(stderr, "type=0x%02x ", pmt_buffer[index]);

								/* video is 1 and 2, audio is 3 and 4, teletext is 5 and 6, dvd subs is .. */
								
								switch(pmt_buffer[index])
									{
									case 1:
									case 2:
										if(debug_flag)
											{
											fprintf(stderr, "type video mpeg2\n"); 
											}
										if(video_pid == 0)
											{
											video_pid = qpid;

//											if(debug_flag)
												{
												fprintf(stderr, "found video pid %d type mpeg2\n", video_pid);
												}
												
											add_pid(qpid);
											}
										break;
									case 3:
									case 4:
										if(debug_flag) fprintf(stderr, "type audio\n");
										if(selected_audio_pid == 0) selected_audio_pid = qpid;

										audio_pid[audio_channels] = qpid;
										audio_channels++;
//										if(debug_flag)
											{
//											fprintf(stderr, "found audio pid %d audio_channels=%d setting selected_audio_pid=%d\n", qpid, audio_channels, selected_audio_pid);
											fprintf(stderr, "found audio pid %d type mp2 ", qpid);
											}

										add_pid(qpid);
										break;
									case 5: 
										switch(pmt_buffer[index + 5]) // private stream type
											{
											case 0x0e: // BBC subtititles
												subtitle_pid[subtitle_channels] = qpid;
												subtitle_channels++;
//												if(debug_flag)
													{
//													fprintf(stderr, "found subtitle pid %d subtitle_channels =%d\n", qpid, subtitle_channels);   
													fprintf(stderr, "found subtitle pid %d\n", qpid);
													}

												add_pid(qpid);
												break;
											case 0x6f: // HbbTV
												hbbtv_pid = qpid;
//												if(debug_flag)
													{
													fprintf(stderr, "found HbbTV pid %d\n", hbbtv_pid); 	
													}

												add_pid(qpid);
												break;
											default:
												fprintf(stderr, "found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
												
												// do not add unknown pids */
												break;	
											} /* end switch pmt_buffer[index + 5] */
										break;
									case 6:
										// private streams
										if(debug_flag) fprintf(stderr, "private stream type=0x%02x ", pmt_buffer[index + 5]);
										switch(pmt_buffer[index + 5]) // private stream type
											{
											case 0x52: // ARD tetelext!
											case 0x56:
											case 0x0e: // BBC teletext
												if(teletext_pid == 0)
													{
													teletext_pid = qpid;
//													if(debug_flag)
														{
														fprintf(stderr, "found teletext pid %d\n", teletext_pid);   
														}

													add_pid(qpid);
													}
												break;
											case 0x59:
												subtitle_pid[subtitle_channels] = qpid;
												subtitle_channels++;
//												if(debug_flag) 
													{
													fprintf(stderr, "found subtitle pid %d subtitle_channels=%d\n", qpid, subtitle_channels);
													}

												add_pid(qpid);
												break;
											case 0x0a: // ZDF astra1 Dolby digital
											case 0x6a:
												if(ac3_pid == 0)
													{
													ac3_pid = qpid;	
//													if(debug_flag)
														{
														fprintf(stderr, "found audio pid %d type ac3\n", qpid);   
														}
														
													add_pid(qpid);
													}
												break;
											/* see en_300468v010601o.pdf */
											case 0x53: // CA_indentifier_descriptor
											case 0x5f: // private data
											case 0x64: // data broadcast
											case 0x65: // scrambling
											case 0x66: // data_broadcast_id
											case 0x69: // PDC_descriptor
											default:
												fprintf(stderr,\
									"found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
												break;

											// do not add unknown pids */
											} /* end switch pmt_buffer[index + 5] */
										break;
//									case 11:
//										switch(pmt_buffer[index + 5]) // private stream type
//											{
//											default:

/*												fprintf(stderr,\
									"found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
*/
//												break;
//											}
//										break;
									case 27: // mpeg4 H264
										if(debug_flag) fprintf(stderr, "type video mpeg4 H264\n"); 
										if(video_pid == 0)
											{
											video_pid = qpid;

//											if(debug_flag)
												{
												fprintf(stderr, "found video pid%d video type mpeg4 H264\n", video_pid);
												}

											add_pid(qpid);
											}
										
										break;
									default:
										fprintf(stderr, "pid=%d pid type=%d (0x%02x), dunno what pid this is.\n",\
										qpid, pmt_buffer[index], pmt_buffer[index]);
										break;
									} /* end switch pid type */

								data_len = ((pmt_buffer[index + 3] & 0x0F) << 8) + pmt_buffer[index + 4];
								if (pmt_buffer[index] == 0x02 || pmt_buffer[index] == 0x03 || pmt_buffer[index] == 0x04)
									{
									i = index + 5;
									while (i < index + 5 + data_len)
										{
										switch (pmt_buffer[i])
											{
										 	case 0x0a:
												if (pmt_buffer[index] == 3 || pmt_buffer[index] == 4)
													{
//													if(debug_flag)
														{
//														fprintf(stderr,	" language %.3s pid %d (0x%x)\n", &pmt_buffer[i + 2], qpid, qpid);
														fprintf(stderr,	"language %.3s\n", &pmt_buffer[i + 2]);
														}
	
													if(memcmp(&pmt_buffer[i + 2], language, 3) == 0)
														{
														selected_audio_pid = qpid;

//														add_pid(qpid);
														}
													}
												break;
											case 0x09:
												ParseCADescriptor(&pmt_buffer[i + 2], pmt_buffer[i + 1]);
												break;
											} /* end switch */

										i += 2 + pmt_buffer[i + 1];
										}
									}
								index += 5 + data_len;
								}

							e = ecminfos;
							for(pa = e; pa != 0; pa = pa->next)
								{
								if(e->ecm_pid)
									{
//									if(debug_flag)
										{
										fprintf(stderr, "found ecm pid %d\n", e->ecm_pid);
										}

									add_pid(e->ecm_pid);
									}
								}	

							if(debug_flag)
								{
								fprintf(stderr, "process_stream(): returning pids complete.\n");
								}

							return 1;
							} /* end if pmt complete */

						} /* end if pid == PMT pid */

					} /* end if have pmt_pid */


				} /* end if ! have_pids_flag */


#ifdef TEST_CODE
			/* test if ecm pid */
			if( (pid != 0) && (pid == e->ecm_pid) )
				{
				/* test if scrambled */
				if( (inptr[5] == 0x80) || (inptr[5] == 0x81) )
					{
					if(parity != (inptr[5] & 1) )
						{
						scrambled_flag = 1;					
						fprintf(stderr, "transmisison seems scrambled\n");
//						exit(1);
						}
					}
				
				} /* end if ecm pid */
#endif // TEST_CODE

			} /* end if inptr[0] == 0x47 */ 
		else /* not a TS packet */
			{
			fprintf(stderr, "Packet %lu - not a TS!\n", count);

			return 0;
			}

		count++;

		inptr += 188;

		packets_in++;
		if(debug_flag)
			{
			fprintf(stderr, "packets_in=%d\n", packets_in);
			}

		loop_cnt++;
		break;


//		if(packets_in == process_packets) break;
		if(packets_in == 1)
			{
			break;
			}

		} /* end while read 188 bytes */


	if(debug_flag)
		{
		fprintf(stderr, "process_stream(): loop_cnt=%d\n", loop_cnt);		
		} 

	if(loop_cnt > 3)
		{
		return 0;
		}
	
	} /* end while fill buffer */


return 1;
} /* end function process_stream */



int add_pid(int pid)
{
if(debug_flag)
	{
	fprintf(stderr, "add_pid(): arg pid=%d, using demux_device_name=%s\n", pid, demux_device_name);
	}

if(pids_open > MAX_PIDS)
	{
	fprintf(stderr, "xdipo: add_pid(): maximum number of pids reached, ignoring more pids.\n");

	return 0;	

	}
fd_demux[pids_open] = pid;
	
fd_demux[pids_open] = open(demux_device_name, O_RDWR);
if(fd_demux[pids_open] < 0)
	{
	fprintf(stderr, "xdipo: add_pid(): open demux device %s failed, aborting.\n", demux_device_name);
	perror("reason ");

	exit(1);
	}

set_ts_filt(fd_demux[pids_open], pid, DMX_PES_OTHER);				

pids_open++;

return 1;
} /* end function add_pid */

