/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


                                  /* SET TABS TO 4 TO READ THIS SOURCE */

#include "xdipo.h"

extern FD_xdipo *fdui;
extern FD_xdipo *fd_main_form;

pthread_t remote_thread;
pthread_attr_t *attributes;



int stop_remote_flag;
int a, c, i;
struct input_keymap_entry *cmd;
struct input_keymap_entry ike;

FILE *fptr;

void *remote_routine()
{
int c;
int data[1024];  
int key_code = 0; // -Wall
int key_pressed = 0; // -Wall

if(debug_flag)
	{
	fprintf(stderr, "remote_routine(): arg none\n");
	}

remote_running_flag = 1;


if(debug_flag)
	{
	fprintf(stderr, "remote_routine():\n");
	}

while(1)
	{
	if(stop_remote_flag)
		{
		break;
		}

//fprintf(stderr, "WAS remote thread\n");

	/*
	read the correct number of bytes from the device for one key event.
	**** NOTE: this number 48 is empirical and works for my Terratec Cinery S2 USB remote, other remotes may perhaps require a different value. ****
	maybe read it with an ioctl from input_keymap_entry? see /usr/include/linux/input.h 
	*/

	while(1)
		{
//fprintf(stderr, "WAS remote inner loop\n");
		if(stop_remote_flag)
			{
			break;
			}		

		for(i = 0; i < 48; i++)
			{
			c = fgetc(fptr);
			if(feof(fptr) )break;
			
			data[i] = c;	
			} /* end for each byte in a field */

		/*
		field 18 has the key value 
		field 20 has the key pressed- released flag
		*/ 
		key_code = data[18];
		key_pressed = data[20];

		if(debug_flag)
			{
			fprintf(stderr, "data=\n");
			for(i = 0; i < 48; i++)
				{
				fprintf(stderr, "%02x ", data[i]);
				}
			fprintf(stderr, "\n");
			}

		/* check for device disconnected */
		if( (data[0] == 0xffffffff) && (data[1] == 0xffffffff) && (data[2] == 0xffffffff) && (data[3] == 0xffffffff) )
			{
			if(debug_flag)
				{
				fprintf(stderr, "read 0xff from ID field, device disconnected?\n");
				}

			fclose(fptr);
			fptr = 0; // use as flag

			break;
			}

		/* these will be checked from idle_cb, so we can set things without crashing the GUI */
		remote_key_code = key_code;

		/* only signal if key down, not if key is released */
		if(key_pressed)
			{
			remote_key_flag = 1;
			}

		} /* end while read fields resulting from a key press */

	usleep(10000);
	} /* end while read remote buffer */

remote_running_flag = 0;

return 0;
} /* end function remote_routine */


int start_remote()
{
int a, c, i, j;    
char temp[1024];
char ir_device[1024];
FILE *pptr;
int found_flag;  

if(debug_flag)
	{
	fprintf(stderr,\
	"start_remote(): using fd_ir_input=%d\n", fd_ir_input);
	}

/* wait for user to plug in USB device */
while(1)
	{
	/*
	looking for something like:
	/dev/input/by-path/pci-0000:04:00.0-event-ir
	going to parse the result of ls in /dev/input/by-path/
	*/
	pptr = popen("ls --color=none /dev/input/by-path/", "r");
	found_flag = 0;
	temp[0] = 0;
	j = 0;
	for(i = 0; i < 1024; i++)
		{
		c = fgetc(pptr);
		if( feof(pptr) )
			{
			ir_device[i] = 0;

			break;
			}	
		if(c == 10) // LF
			{
			temp[j] = 0;
			j = 0;
			if(strstr(temp, "event-ir") != 0)
				{
				if(debug_flag)
					{
					fprintf(stderr, "found it, i=%d temp=%s\n", i, temp);
					}
				found_flag = 1;

				break;
				}
			j = 0;				
			}
		else
			{
			temp[j] = c;
			j++;
			}
		}
	pclose(pptr);

	if(! found_flag)
		{
		if(debug_flag)
			{
			fprintf(stderr, "could not find event-ir device\n"); 
			}
		}
	else // found
		{
		if(debug_flag)
			{
			fprintf(stderr, "temp=%s\n", temp);
			}

		sprintf(ir_device, "/dev/input/by-path/%s", temp);
		if(debug_flag)
			{
			fprintf(stderr, "trying ir_device=%s\n", ir_device);
			}
		
		fptr = fopen(ir_device, "r");
		
		if(! fptr)
			{
			fprintf(stderr, "could not open /dev/input/event13 for read,aborting.\n");
		
			return 0;
			}
	
		if(debug_flag)
			{
			fprintf(stderr, "fptr=%p\n", fptr);
			}
		
		/* grab the device for us alone, this prevents output to other applications, as that would cause havoc */
		a = ioctl(fileno(fptr), EVIOCGRAB);
		if(a < 0)
			{
			fprintf(stderr, "ioctl EVIOCGRAB failed because ");
			perror("");
		
			return 0;
			}
				
		break;
		} /* end else if found */

	if(debug_flag)
		{
		fprintf(stderr, "waiting for user to plug in USB device.\n");
		}

	/* prevent 100 % processor load */
	usleep(1000000);
	} /* end while wait for IR USB device to be pluuged in */


stop_remote_flag = 0;
remote_running_flag = 0;

pthread_create(&remote_thread, NULL, &remote_routine, NULL);

if(debug_flag)
	{
	fprintf(stderr, "start_remote(): remote_started\n");
	}

return 1;
} /* end function start_remote */


int stop_remote()
{ 
if(debug_flag)
	{
	fprintf(stderr, "stop_remote(): arg none\n");	
	}

stop_remote_flag = 1;

while(remote_running_flag)
	{
	usleep(100000);
	}

fclose(fptr);

if(debug_flag)
	{
	fprintf(stderr, "stop_remote(): renote thread stopped\n");
	}

return 1;
} /* end function stop_remote */



int do_keys(int key_code, int key_pressed)
{
int selected_line;
time_t now;
char temp[4096];
char *ptr;
char dummy[80];
struct tm *local_time;
char path_filename[4096];

if(debug_flag)
	{
	fprintf(stderr, "do_keys(): arg key_code=%d key_pressed=%d\n", key_code, key_pressed);
	}

if(! remote_running_flag) return 1;

/*
do your things here, this can be used to do anything on a key press and key release, just some silly examples

Terratec Cinergy S2 USB IR remote key codes:
mute						0x71
EPG							0x79						
iconify						0x91
power						0x01
1							0x02
2							0x03
3							0x04
4							0x05
5							0x06
6							0x07
7							0x08
8							0x09  
9							0x0a
0							0x0b
cursor up					0x67
ESC							0x9e
snapshot					0x8e
cursor left					0x69
OK							0x60
cursor right				0x6a	
channel up					0x73
volume up					0x6d
cursor down					0x6c
record						0xa7
channel down				0x72
volume down					0x8b
pause/play					0x8f
stop						0x80
*/

if(key_pressed)
	{
	if(debug_flag)
		{
		fprintf(stderr, "key %02x pressed\n", key_code);
		}

	switch(key_code)
		{
		case 0x71: // mute 
			popen("/usr/local/sbin/cinergy_mute_in", "w");
			break;
		case 0x79: // EPG
			popen("/usr/local/sbin/cinergy_epg_in &", "w");
			break;
		case 0x91: // iconify
			popen("/usr/local/sbin/cinergy_iconify_in &", "w");
			break;
		case 0x01: // power
//			popen("/usr/local/sbin/cinergy_power_in &", "w");
			/* test if user abort from main form record button */
			if(recorder_mode & RECORDER_MAN_RECORD_MODE)
				{
				/* if stopping manually started recording leave the timers unchanged */ 			

				close_output_files();

				recorder_mode = RECORDER_STOP_MODE;
				}
			else
				{	
				recorder_mode = RECORDER_ABORT_MODE; // timers and record loop will see this, and close files, then change to RECODER_STOP_MODE.	
				}

			exit(0);
			break;
		case 0x02: // 1
			popen("/usr/local/sbin/cinergy_1_in &", "w");
			break;
		case 0x03: // 2
			popen("/usr/local/sbin/cinergy_2_in &", "w");
			break;
		case 0x04: // 3
			popen("/usr/local/sbin/cinergy_3_in &", "w");
			break;
		case 0x05: // 4
			popen("/usr/local/sbin/cinergy_4_in &", "w");
			break;
		case 0x06: // 5
			popen("/usr/local/sbin/cinergy_5_in &", "w");
			break;
		case 0x07: // 6
			popen("/usr/local/sbin/cinergy_6_in &", "w");
			break;
		case 0x08: // 7
			popen("/usr/local/sbin/cinergy_7_in &", "w");
			break;
		case 0x09: // 8
			popen("/usr/local/sbin/cinergy_8_in &", "w");
			break;
		case 0x0a: // 9
			popen("/usr/local/sbin/cinergy_9_in &", "w");
			break;
		case 0x0b: // 0
			popen("/usr/local/sbin/cinergy_9_in &", "w");
			break;
		case 0x67: // cursor up
			popen("/usr/local/sbin/cinergy_cuup_in &", "w");
			break;
		case 0x9e: // ESC
			popen("/usr/local/sbin/cinergy_esc_in &", "w");
			break;
		case 0x8e: // snap shot
			popen("/usr/local/sbin/cinergy_snap_in &", "w");
			break;
		case 0x69: // cursor left
			popen("/usr/local/sbin/cinergy_culeft_in &", "w");
			break;
		case 0x60: // OK
//			popen("/usr/local/sbin/cinergy_OK_in &", "w");
			if(! select_stream(selected_stream) ) return 0;
			break;
		case 0x6a: // cursor right
			popen("/usr/local/sbin/cinergy_curight_in &", "w");
			break;
		case 0x73: // channel up
//			popen("/usr/local/sbin/cinergy_chup_in &", "w");
			if(selected_stream > 0) selected_stream--;
			break;
		case 0x6d: // volume up
			popen("/usr/local/sbin/cinergy_volup_in &", "w");
			break;
		case 0x6c: // cursor down
			popen("/usr/local/sbin/cinergy_cudown_in &", "w");
			break;
		case 0xa7: // record
//			popen("/usr/local/sbin/cinergy_record_in &", "w");

			/* test if already recording */
			if(recorder_mode & RECORDER_RECORD_MODE)
				{
				return 0;
				}

			selected_line = fl_get_browser(fdui -> main_form_stream_browser);
			if(! selected_line)
				{
				fprintf(stderr, "no entry in browser selected.\n");

				return 0;
				}

			selected_stream = abs(selected_line);

			/* select stream */
			if(! select_stream( selected_stream ) )
				{
				fprintf(stderr, "no entry in browser selected.\n");
		
				return 0;
				}
	
			/* and move the dish to the sat or angle */
			main_form_goto_button_cb(0, 0);

			/* going to compose a filename */
			ptr = (char *)fl_get_browser_line(fdui -> main_form_stream_browser, selected_stream);
			if(! ptr)
				{
				fprintf(stderr, "do_keys(): fl_get_brower_line() failed for selected_stream=%d, using time_t format filename\n", selected_stream);

				now = time(0);
				sprintf(temp, "%s/%lu.ts", output_dir, now);
				}
			else
				{
				/* format is
				2 12515.25 h 22000 -16.80 Astra1_NL -19.20 8192 S QPSK AUTO 0.00
				*/
				sscanf(ptr, "%s %s", dummy, temp); // skip format string, use only fits argument before ':'
//fprintf(stderr, "ptr=%s\n", ptr);

				/* cut string at first ':' */
				ptr = strchr(temp, ':');
				if(ptr) *ptr = 0;

				now = time(0);
				local_time = localtime(&now);
				strftime(dummy, 511, "%Y_%b_%d_%H_%M_%S", local_time);				

//fprintf(stderr, "temp=%s\n", temp);
				
				snprintf(path_filename, 4095, "%s/%s_%s.ts", output_dir, temp, dummy);
				}			

			if(output_filename) free(output_filename);
			output_filename = strsave(path_filename);
			if(! output_filename)
				{
				fprintf(stderr, "xdipo: main_form_output_filename_input_field_cb(): could not allocate space for output_filename, aborting.\n");

				return 0;	
				}

			if(! open_output_files() )
				{
				return 0;
				}

			/* if service_id specified read fd_dvr for PAT 0 to find PMT and other service_id related pids */
			if(use_filter_flag)  /* set by parse_pids(), that is called by select_stream() */
				{
				/* read fd_dvr for PAT 0 to find PMT and other service_id related pids */
				if(! process_stream(fd_dvr, service_id) )
					{
					fprintf(stderr, "xdipo: main(): could not find program related pids for service_id=%d, aborting.\n", service_id);
	
					return 0;
					}

				} /* end if use_filter_flag */


			set_record_button(1);

			recording_start_time = time(NULL);

			recorder_mode = RECORDER_RECORD_MODE;
			recorder_mode |= RECORDER_MAN_RECORD_MODE;	// indicate we are recording manually, not from timers, timer need to know this when releasing record button for stop..

			break;
		case 0x72: // channel down
//			popen("/usr/local/sbin/cinergy_chdown_in &", "w");
			selected_stream++;
			break;
		case 0x8b: // volume down
			popen("/usr/local/sbin/cinergy_voldown_in &", "w");
			break;
		case 0x8f: // pause / play
			popen("/usr/local/sbin/cinergy_pauseplay_in &", "w");
			break;
		case 0x80: // stop
//			popen("/usr/local/sbin/cinergy_stop_in &", "w");
			/* test if user abort from main form record button */
			if(recorder_mode & RECORDER_MAN_RECORD_MODE)
				{
				/* if stopping manually started recording leave the timers unchanged */ 			

				close_output_files();

				recorder_mode = RECORDER_STOP_MODE;
				}
			else
				{	
				recorder_mode = RECORDER_ABORT_MODE; // timers and record loop will see this, and close files, then change to RECODER_STOP_MODE.	
				}

			set_record_button(0);
	
			/* enable all controls on all forms again, if it not already */
			enable_controls(1);
			break;
		} /* end switch */

	} /* end if key pressed */
else // key released
	{		
	if(debug_flag)
		{
		fprintf(stderr, "key %02x released\n", key_code);		
		}
				
	switch(key_code)
		{
		}

	} /* end if key released */

return 1;
} /* end function do_keys */

