/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

                                  /* SET TABS TO 4 TO READ THIS SOURCE */

/*
if you use the Terratec Cinergy S2 USB HD receiver, it often just stops outputting data after
a station change, or any use basically.
This powers down and then powers up the USB device 2 times and then it seems to work again.
The device must be connected to the same USB port to work,
and no other thing such as for example hubs should be connected to that port.
*/


#ifndef FD_xdipo_h_
#define FD_xdipo_h_

#include <forms.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/time.h>
#include <signal.h>
#include <pwd.h>
#include <math.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/poll.h>
#include <stdint.h>
#include <locale.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/io.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>

/* for IR remote thread */
#include <pthread.h>
#include <linux/input.h>
#include "program_list.h"
#include "timers.h"

typedef struct {
	FL_FORM *main_form;
	FL_FORM *installer_commands_form;	
	FL_FORM *timer_form;

	FL_OBJECT *main_form_title;
	FL_OBJECT *main_form_recorder_menu;
	FL_OBJECT *main_form_frequency_input_field;
	FL_OBJECT *main_form_polarization_input_field;
	FL_OBJECT *main_form_symbolrate_input_field;
	FL_OBJECT *main_form_type_input_field;
	FL_OBJECT *main_form_modulation_input_field;
	FL_OBJECT *main_form_fec_input_field;
	FL_OBJECT *main_form_rolloff_input_field;
	FL_OBJECT *main_form_stream_browser;
	FL_OBJECT *main_form_add_stream_button;
	FL_OBJECT *main_form_delete_stream_button;
	FL_OBJECT *main_form_fontsize_up_button;
	FL_OBJECT *main_form_fontsize_down_button;
	FL_OBJECT *main_form_nudge_motor_west_button;
	FL_OBJECT *main_form_nudge_motor_east_button;
	FL_OBJECT *main_form_nudge_tilt_motor_up_button;
	FL_OBJECT *main_form_nudge_tilt_motor_down_button;
	FL_OBJECT *main_form_east_text;
	FL_OBJECT *main_form_west_text;
	FL_OBJECT *main_form_increase_nudge_size_button;
	FL_OBJECT *main_form_decrease_nudge_size_button;
	FL_OBJECT *main_form_toggle_step_timed_mode_button;
	FL_OBJECT *main_form_toggle_goto_angle_sattelite_button;
	FL_OBJECT *main_form_store_present_satellite_number_button;
	FL_OBJECT *main_form_goto_button;
	FL_OBJECT *main_form_increment_satellite_number_button;
	FL_OBJECT *main_form_decrement_satellite_number_button;
	FL_OBJECT *main_form_satellite_number_input_field;
	FL_OBJECT *main_form_angle_input_field;
	FL_OBJECT *main_form_angle_down_button;
	FL_OBJECT *main_form_angle_up_button;
	FL_OBJECT *main_form_search_input_field;
	FL_OBJECT *main_form_search_button;
	FL_OBJECT *main_form_case_sensitive_button;
	FL_OBJECT *main_form_step_value_input_field;
	FL_OBJECT *main_form_signal_strength_indicator;
	FL_OBJECT *main_form_signal_strength_text_field;
	FL_OBJECT *main_form_signal_strength_text_text_field;
	FL_OBJECT *main_form_carrier_output_field;
	FL_OBJECT *main_form_viterbi_output_field;
	FL_OBJECT *main_form_sync_output_field;
	FL_OBJECT *main_form_lock_output_field;
	FL_OBJECT *main_form_bit_error_rate_text_text_field;
	FL_OBJECT *main_form_bit_error_rate_text_field;
	FL_OBJECT *main_form_beep_button;
	FL_OBJECT *main_form_beep_select_button;
	FL_OBJECT *main_form_signal_strength_slider;
	FL_OBJECT *main_form_installer_button;
	FL_OBJECT *main_form_elevation_output_field;
	FL_OBJECT *main_form_elevation_output_field_text;
	FL_OBJECT *main_form_satellite_name_input_field;
	FL_OBJECT *main_form_satellite_location_input_field;
	FL_OBJECT *main_form_longitude_input_field;
	FL_OBJECT *main_form_latitude_input_field; 
	FL_OBJECT *main_form_local_clock;
	FL_OBJECT *main_form_command_output_field;
	FL_OBJECT *main_form_command_output_field_text;
	FL_OBJECT *main_form_output_filename_output_field;
	FL_OBJECT *main_form_output_filename_output_field_text;
	FL_OBJECT *main_form_bitrate_output_field;
	FL_OBJECT *main_form_bitrate_output_field_text;
	FL_OBJECT *main_form_pids_input_field;
	FL_OBJECT *main_form_seek_slider;
	FL_OBJECT *main_form_record_button;
	FL_OBJECT *main_form_enable_remote_button;
	FL_OBJECT *main_form_exit_switch;

	FL_OBJECT *installer_commands_form_form_title;
	FL_OBJECT *installer_commands_form_recalculate_positions_button;
	FL_OBJECT *installer_commands_form_satellite_number_input_field;
	FL_OBJECT *installer_commands_form_recalculate_param1_input_field;
	FL_OBJECT *installer_commands_form_recalculate_param2_input_field;
	FL_OBJECT *installer_commands_form_start_motor_driving_west_button;
	FL_OBJECT *installer_commands_form_halt_motor_button;
	FL_OBJECT *installer_commands_form_start_motor_driving_east_button;
	FL_OBJECT *installer_commands_form_set_west_limit_button;
	FL_OBJECT *installer_commands_form_disable_limits_button;
	FL_OBJECT *installer_commands_form_set_east_limit_button;
	FL_OBJECT *installer_commands_form_goto_reference_position_button;
	FL_OBJECT *installer_commands_form_enable_limits_button;
	FL_OBJECT *installer_commands_form_close_button;
//	FL_OBJECT *
//	FL_OBJECT *

	FL_OBJECT *timer_form_title;
	FL_OBJECT *timer_form_browser;
	FL_OBJECT *timer_form_directory_input_field;
	FL_OBJECT *timer_form_filename_output_field;
	FL_OBJECT *timer_form_filename_output_field_text;  
	FL_OBJECT *timer_form_start_text_field;
	FL_OBJECT *timer_form_stop_text_field;
	FL_OBJECT *timer_form_start_year_input_field;
	FL_OBJECT *timer_form_start_month_input_field;
	FL_OBJECT *timer_form_start_day_input_field;
	FL_OBJECT *timer_form_start_hour_input_field;
	FL_OBJECT *timer_form_start_minute_input_field;
	FL_OBJECT *timer_form_start_weekday_output_field_text;
	FL_OBJECT *timer_form_start_weekday_output_field_text_text;
	FL_OBJECT *timer_form_length_input_field;
	FL_OBJECT *timer_form_showview_output_field;
	FL_OBJECT *timer_form_showview_output_field_text;
	FL_OBJECT *timer_form_stop_year_input_field;
	FL_OBJECT *timer_form_stop_month_input_field;
	FL_OBJECT *timer_form_stop_day_input_field;
	FL_OBJECT *timer_form_stop_hour_input_field;
	FL_OBJECT *timer_form_stop_minute_input_field;
	FL_OBJECT *timer_form_stop_weekday_output_field_text;
	FL_OBJECT *timer_form_daily_button;
	FL_OBJECT *timer_form_work_days_button;
	FL_OBJECT *timer_form_weekly_button;
	FL_OBJECT *timer_form_delete_button;
//	FL_OBJECT *timer_form_add_button;
	FL_OBJECT *timer_form_cancel_button;
	FL_OBJECT *timer_form_accept_button;
	FL_OBJECT *timer_form_fontsize_up_button;
	FL_OBJECT *timer_form_fontsize_down_button;

} FD_xdipo;

extern FD_xdipo *create_form_main_form(void);
extern FD_xdipo *create_form_installer_commands_form(void);
extern FD_xdipo *create_form_timer_form(void);

#define READSIZE			65535
#define TEMP_SIZE			65535

/* this is set by the the length limit of a browser line in xforms */
#define TEXT_ARRAY_SIZE		1024

/* set some limits for this system */
#define MAX_H_PIXELS		2048
#define MAX_V_PIXELS		2048
#define MAX_SCREEN_LINES	200

/* for main form recorder menu */
#define RECORDER_MENU_TIMERS		1

/* for horizontal motion positioner diseqc */
#define TIME_MODE				0
#define STEP_MODE				1

/* for signal indicators */
#define MAX_SNR_SCALE			20.0
#define BEEP_MINIMUM_FREQ		200
#define BEEP_MAXIMUM_FREQ		2000

/* for position browser display */
#define SHOW_TOP				0
#define SHOW_SAME_POSITION		1
#define SHOW_UNREAD_AT_TOP		2
#define SHOW_BOTTOM				3
#define SHOW_SELECTED_IN_MIDDLE	4

/* use angle or satellite */
#define GOTO_ANGLE				0
#define GOTO_SATELLITE			1

#define MAX_PIDS	16

/* for recorder */
#define RECORDER_STOP_MODE			0
#define RECORDER_RECORD_MODE		1
#define RECORDER_PLAYBACK_MODE		2
#define RECORDER_ABORT_MODE			4
#define RECORDER_MAN_RECORD_MODE	8

/* for timers (use 1, 2 ,4 ,8 etc.. ) */
#define TIMER_REPEAT_DAILY			1
#define TIMER_REPEAT_WEEKLY			2
#define TIMER_RUNNING				4
#define TIMER_EXPIRED				8
#define TIMER_OVERLAP				16
#define TIMER_ERROR					32
#define TIMER_REPEAT_WORK_DAYS		64
#define TIMER_RUNNING_CANCELLED		128
#define TIMER_REPEAT_NO				256
#define TIMER_SLEEPING				512

/* times trying to move dish to requested position when no signal lock */
#define MAX_POSITIONER_TRIES 10

/* dvr to out */
#define DVR_BUFFER_SIZE (100*188*1024)	// 19 251 20	19 MB 
#define DMX_BUFFER_SIZE 6400*1024		//  6 553 600	 6 MB

/* signalling recorder trhead to idle_cb */
int recorder_error_mode;
#define RECORDER_NO_ERROR		0
#define RECORDER_OVERFLOW		1	
#define RECORDER_WRITE_ERROR	2
#define RECORDER_DISK_FULL		3


int debug_flag;
char *home_dir;
char *user_name;
int gui_flag;

int commands_form_recalculate_param1;
int commands_form_recalculate_param2;
int8_t nudge_size;
int satellite_number, step_time_mode;
char *frontend_device_name, *demux_device_name, *dvr_device_name, *output_filename;
int fd_frontend, fd_dvr;
int fd_demux[MAX_PIDS];
FILE *ofptr;
int beep_mode;
double frequency;
int symbolrate;
int polarization;
char type[20];
char modulation[10];
char fec[10];
double rolloff;
double angle, elevation;
int selected_stream;
int old_selected_stream;
char *bit_errors_log_filename;
FILE *fpbe;
int goto_mode;
int root_flag;
double latitude, longitude, satellite_position, calculated_angle;
fd_set active_fd_set, read_fd_set;
char *output_dir;
time_t recording_start_time;
char pids_string[80];
int pids_open;
int main_form_recorder_menu_selection;

int main_form_fontsize;
int timer_form_fontsize;
int some_form_open_flag;
struct timer *selected_timer_pa;
struct timer *running_timer;
int timer_overlap_flag;                                                          
int timer_form_shown_flag;
char *record_filename;
char *timer_form_record_filename;
int selected_program;
int error_message_flag;
int recorder_mode;
int recorder_pause_flag;
int carrier_flag, viterbi_flag, sync_flag, lock_flag;
int seconds_to_wait;
struct pollfd pfds[1];
time_t old_time;
double total_bytes_read;
double total_bitrate;
double total_seconds;
int use_filter_flag;
int service_id;
FILE *jpinfo_ptr;
unsigned char *input_buffer;
int case_sensitive_flag;
int dsp_beep_flag;

int remote_running_flag;
char *ir_input_device_name;
int fd_ir_input;

int remote_key_flag;
int remote_key_code;

int max_bytes;
int enable_remote_flag;


int process_packets;
#define PROCESS_PACKETS                 1000

/* this last so proto knows about structure definitions etc. */
#include "xdipo_proto.h"


#define XDIPO_VERSION "0.8.5"

#endif /* FD_xdipo_h_ */

